from __future__ import division

from numpy.testing import assert_allclose

from numpy import exp

from optimix import check_grad

from limix_inference.cov import EyeCov
from limix_inference.fruits import Oranges
from limix_inference.fruits import Apples


def test_eye_value():
    cov = EyeCov()
    cov.scale = 2.1
    o = Oranges(None)
    assert_allclose(2.1, cov.value(o, o))


def test_eye_gradient_1():
    cov = EyeCov()
    cov.scale = 2.1
    a = Apples(None)
    cov.set_data((a, a))

    def func(x):
        cov.scale = exp(x[0])
        return cov.feed().value()

    def grad(x):
        cov.scale = exp(x[0])
        return cov.feed().gradient()

    assert_allclose(check_grad(func, grad, [0.1]), 0, atol=1e-7)


def test_eye_gradient_2():
    cov = EyeCov()
    cov.scale = 2.1
    a = Apples(5)
    cov.set_data((a, a))

    def func(x):
        cov.scale = exp(x[0])
        return cov.feed().value()

    def grad(x):
        cov.scale = exp(x[0])
        return cov.feed().gradient()

    assert_allclose(check_grad(func, grad, [0.1]), 0, atol=1e-7)


def test_eye_gradient_3():
    cov = EyeCov()
    cov.scale = 2.1
    a = Apples(5)
    o = Oranges(4)
    cov.set_data((a, o))

    def func(x):
        cov.scale = exp(x[0])
        return cov.feed().value()

    def grad(x):
        cov.scale = exp(x[0])
        return cov.feed().gradient()

    assert_allclose(check_grad(func, grad, [0.1]), 0, atol=1e-7)


if __name__ == '__main__':
    __import__('pytest').main([__file__, '-s'])



"""
**************
Mean functions
**************

- :class:`.LinearMean`
- :class:`.OffsetMean`
- :class:`.SumMean`

Linear mean
^^^^^^^^^^^

.. autoclass:: LinearMean
  :members:

Offset mean
^^^^^^^^^^^

.. autoclass:: OffsetMean
  :members:

Sum mean
^^^^^^^^

.. autoclass:: SumMean
  :members:
"""

from ._offset import OffsetMean
from ._linear import LinearMean
from ._sum import SumMean



from __future__ import division

import numpy as np
from numpy import ones
from numpy.testing import assert_allclose

from numpy_sugar.linalg import economic_qs_linear

from limix_inference.lmm import FastLMM
from limix_inference.fruits import Apples
from limix_inference.lik import DeltaProdLik
from limix_inference.cov import LinearCov
from limix_inference.cov import EyeCov
from limix_inference.cov import SumCov
from limix_inference.mean import OffsetMean
from limix_inference.random import GLMMSampler


def test_learn():
    random = np.random.RandomState(9458)
    N = 50
    X = random.randn(N, N + 1)
    X -= X.mean(0)
    X /= X.std(0)
    X /= np.sqrt(X.shape[1])
    offset = 1.0

    mean = OffsetMean()
    mean.offset = offset
    mean.set_data(N, purpose='sample')

    cov_left = LinearCov()
    cov_left.scale = 1.5
    cov_left.set_data((X, X), purpose='sample')

    cov_right = EyeCov()
    cov_right.scale = 1.5
    cov_right.set_data((Apples(N), Apples(N)), purpose='sample')

    cov = SumCov([cov_left, cov_right])

    lik = DeltaProdLik()

    y = GLMMSampler(lik, mean, cov).sample(random)

    (Q0, Q1), S0 = economic_qs_linear(X)

    flmm = FastLMM(y, Q0, Q1, S0, covariates=ones((N, 1)))

    flmm.learn()

    assert_allclose(flmm.beta[0], 0.709180072285, rtol=1e-5)
    assert_allclose(flmm.genetic_variance, 2.26638555117, rtol=1e-5)
    assert_allclose(flmm.environmental_variance, 1.02391614253, rtol=1e-5)


#
# def test_predict_1():
#     random = np.random.RandomState(228)
#     N = 50
#     X = random.randn(N, N+1)
#
#     offset = 1.2
#     scale = 3.0
#     delta = 0.5
#     y = FastLMMSampler(offset, scale, delta, X).sample(random)
#
#     X = DesignMatrixTrans(X).transform(X)
#     (Q0, Q1), S0 = economic_qs_linear(X)
#
#
#     flmm = FastLMM(y, Q0, Q1, S0, covariates=ones((N, 1)))
#     flmm.learn()
#     assert_allclose(flmm.predict(X, ones((N, 1)), X).logpdf(y), -54.1934992524)
#
#
# def test_predict_2():
#     random = np.random.RandomState(228)
#     N = 50
#     X = random.randn(N, N+1)
#
#     offset = 1.2
#     scale = 3.0
#     delta = 0.5
#     y = FastLMMSampler(offset, scale, delta, X).sample(random)
#
#     X = DesignMatrixTrans(X).transform(X)
#     (Q0, Q1), S0 = economic_qs_linear(X)
#
#     flmm = FastLMM(y, Q0, Q1, S0, covariates=ones((N, 1)))
#     flmm.learn()
#     p = flmm.predict(X, ones((N, 1))[5, :], X[5, :])
#     y5 = y[5]
#     y6 = y[6]
#     assert_allclose(p.logpdf(y5), -1.02552843174, rtol=1e-5)
#     assert_allclose(p.logpdf(y6), -5.39355862337, rtol=1e-5)

if __name__ == '__main__':
    __import__('pytest').main([__file__, '-s'])



from __future__ import division

from numpy import array, dot, empty, ones, sqrt
from numpy.random import RandomState
from numpy.testing import assert_almost_equal, assert_allclose

from numpy_sugar.linalg import economic_qs_linear

from limix_inference.glmm import ExpFamEP
from limix_inference.lik import BernoulliProdLik
from limix_inference.link import LogitLink


def test_bernoulli_lml():
    n = 3
    M = ones((n, 1)) * 1.
    G = array([[1.2, 3.4], [-.1, 1.2], [0.0, .2]])
    (Q, S0) = economic_qs_linear(G)
    y = array([1., 0., 1.])
    lik = BernoulliProdLik(LogitLink())
    lik.outcome = y
    ep = ExpFamEP(lik, M, Q[0], Q[1], S0 + 1.0)
    ep.beta = array([1.])
    assert_almost_equal(ep.beta, array([1.]))
    ep.v = 1.
    ep.delta = 0.
    assert_almost_equal(ep.lml(), -2.3202659215368935)
    assert_almost_equal(ep.sigma2_epsilon, 0)
    assert_almost_equal(ep.sigma2_b, 1)


def test_bernoulli_gradient_over_v():
    n = 3
    M = ones((n, 1)) * 1.
    G = array([[1.2, 3.4], [-.1, 1.2], [0.0, .2]])
    (Q, S0) = economic_qs_linear(G)
    y = array([1., 0., 1.])
    lik = BernoulliProdLik(LogitLink())
    lik.outcome = y
    ep = ExpFamEP(lik, M, Q[0], Q[1], S0 + 1.0)
    ep.beta = array([1.])
    assert_almost_equal(ep.beta, array([1.]))
    ep.v = 1.
    ep.delta = 0.

    analytical_gradient = ep._gradient_over_v()

    lml0 = ep.lml()
    step = 1e-5
    ep.v = ep.v + step
    lml1 = ep.lml()

    empirical_gradient = (lml1 - lml0) / step

    assert_almost_equal(empirical_gradient, analytical_gradient, decimal=4)


def test_bernoulli_optimize():
    random = RandomState(139)
    nsamples = 100
    nfeatures = nsamples + 10

    G = random.randn(nsamples, nfeatures) / sqrt(nfeatures)

    M = ones((nsamples, 1))

    u = random.randn(nfeatures)

    z = 0.1 + dot(G, u) + 0.5 * random.randn(nsamples)

    y = empty(nsamples)
    y[z > 0] = 1
    y[z <= 0] = 0

    (Q, S0) = economic_qs_linear(G)

    lik = BernoulliProdLik(LogitLink())
    lik.outcome = y
    ep = ExpFamEP(lik, M, Q[0], Q[1], S0)
    ep.optimize()
    assert_allclose(ep.lml(), -67.68161933698035, rtol=1e-5)
    assert_allclose(ep.heritability, 0.952635979147327, rtol=1e-5)
    assert_allclose(ep.beta[0], -0.15856727011101007, rtol=1e-5)


if __name__ == '__main__':
    __import__('pytest').main([__file__, '-s'])
